"""
Created on April 23, 2022
Reference: "Deep Interest Network for Click-Through Rate Prediction", KDD, 2018
@author: Mincai Lai, laimincai@shanghaitech.edu.cn
"""
import torch

from torch_ctr.layers import FeaturesEmbedding, SequenceFeaturesEmbedding, MultiLayerPerceptron
from torch.nn import ModuleList, Sequential, Linear,  PReLU
from ..activation import Dice

class DIN(torch.nn.Module):
    def __init__(self, sequence_field_dims, sparse_field_dims, embed_dim, mlp_dims=None, attention_mlp_dims=[36], activation='dice', dropout=None):
        super(DIN, self).__init__()

        self.num_seq_fields = len(sequence_field_dims)
        self.embedding_sequence = SequenceFeaturesEmbedding(sequence_field_dims, embed_dim, mode="concat")
        self.embedding_sparse = FeaturesEmbedding(sparse_field_dims, embed_dim)
        self.embed_output_dim = (len(sparse_field_dims) + 2*self.num_seq_fields)*embed_dim
        self.mlp = MultiLayerPerceptron(self.embed_output_dim, mlp_dims, dropout, activation="dice")
        self.attention_layers = ModuleList([
            ActivationUnit(embed_dim, attention_mlp_dims, activation, use_softmax=False) for i in range(self.num_seq_fields)]) 

    def forward(self, x_sequence, x_candidate, x_sparse):
        """
        The original paper without dense feature.
        :param x_sequence: Long tensor of size ``(batch_size, num_seq_fields, seq_length)``
                        # each fields is a tensor list
        :param x_candidate: Long tensor of size ``(batch_size, num_seq_fields)``
                        # each fields is a candidate field id for x_sequence
        :param x_sparse: Long tensor of size ``(batch_size, num_sparse_fields)``
                        # sparse: user_profile_feature, candidate_feature, context_feature
        """
        #embed_x_sequence: (batch_size, num_seq_fields, seq_length, emb_dim) 
        #embed_x_candidate:  (batch_size, num_seq_fields, emb_dim) 
        embed_x_sequence, embed_x_candidate = self.embedding_sequence(x_sequence, x_candidate)
        embed_x_sparse = self.embedding_sparse(x_sparse) #(batch_size, num_sparse_fields, embed_dim)
        pooling_concat = []
        for i in range(self.num_seq_fields):
            #为每一个序列特征都训练一个独立的激活单元 eg.历史序列item id，历史序列item id品牌
            pooling_seq = self.attention_layers[i](embed_x_sequence[:,i,:,:], embed_x_candidate[:,i,:])
            pooling_concat.append(pooling_seq.unsqueeze(1)) #(batch_size, 1, emb_dim)
        pooling_concat = torch.cat(pooling_concat, dim=1)  #(batch_size, num_seq_fields, emb_dim)
        
        mlp_in = torch.cat([pooling_concat.flatten(start_dim=1), embed_x_candidate.flatten(start_dim=1), embed_x_sparse.flatten(start_dim=1)], dim=1) #(batch_size, N)

        y = self.mlp(mlp_in)
        return torch.sigmoid(y.squeeze(1))


class ActivationUnit(torch.nn.Module):
    """
        DIN Attention Layer
    """
    def __init__(self, emb_dim, attention_mlp_dims=[36], activation="dice", use_softmax=False):
        super(ActivationUnit, self).__init__()
        self.emb_dim = emb_dim
        self.use_softmax = use_softmax 
        self.attention = MultiLayerPerceptron(4 * self.emb_dim, attention_mlp_dims, activation=activation)

    def forward(self, history, candidate):
        """
        :param history: Long tensor of size ``(batch_size, seq_length, emb_dim) ``
        :param candidate: Long tensor of size ``(batch_size, emb_dim)``
        """
        seq_length = history.size(1)
        candidate = candidate.unsqueeze(1).expand(-1, seq_length, -1) #batch_size,seq_length,emb_dim 
        att_input = torch.cat([candidate, history, candidate - history, candidate * history], dim=-1) # batch_size,seq_length,4*emb_dim 
        att_weight = self.attention(att_input.view(-1, 4*self.emb_dim))  #  #(batch_size*seq_length,4*emb_dim)
        att_weight = att_weight.view(-1, seq_length)    #(batch_size*seq_length, 1) -> (batch_size,seq_length)
        if self.use_softmax:
            att_weight = att_weight.softmax(dim=-1) 
        # (batch_size, seq_length, 1) * (batch_size, seq_length, emb_dim)
        output = (att_weight.unsqueeze(-1) * history).sum(dim=1) #(batch_size,emb_dim)
        return output


"""
Created on April 22, 2022
Reference: "Wide & Deep Learning for Recommender Systems", DLRS, 2016
@author: Mincai Lai, laimincai@shanghaitech.edu.cn
"""

import torch

from ..layers import FeaturesLinear, MultiLayerPerceptron, FeaturesEmbedding

class WideAndDeep(torch.nn.Module):

    def __init__(self, dense_field_nums, sparse_field_dims, embed_dim, mlp_dims, dropout):
        super().__init__()
        self.linear = FeaturesLinear(dense_field_nums)
        self.embedding = FeaturesEmbedding(sparse_field_dims, embed_dim)
        self.mlp = MultiLayerPerceptron(len(sparse_field_dims) * embed_dim, mlp_dims, dropout)

    def forward(self, x_dense, x_sparse):
        """
        :param x_dense: Long tensor of size ``(batch_size, num_dense_fields)``
        :param x_sparse: Long tensor of size ``(batch_size, num_sparse_fields)``
        """
        y_wide = self.linear(x_dense)
        embed_x = self.embedding(x_sparse)
        y_deep = self.mlp(embed_x.flatten(start_dim=1))
        y = y_wide + y_deep
        y = torch.sigmoid(y.squeeze(1))
        return y




import numpy as np
from torch.utils.data import Dataset
import torch
import pickle


class AmazonDataset(Dataset):
    """

    """
    def __init__(self, data_path="./amazon/data/amazon_dict.pkl"):
        with open(data_path, "rb") as f:
            data = pickle.load(f)
        sequence_field_dims = np.zeros(2, dtype=np.int32)
        for i in range(2):
            sequence_field_dims[i] = np.unique(data["candicate"][:,i]).size

        self.max_len = 50
        self.sparse_field_dims = sequence_field_dims
        self.sequence_field_dims = sequence_field_dims
        
        self.x_sparse = data["candicate"] #统一接口 先多train两个特征
        self.x_sequence = data["history"]   #batch_size,n_seq_fields,seq_len
        self.x_candidate = data["candicate"] #batch_size,n_seq_fields

        self.length = self.x_candidate.shape[0]
        self.y = data["label"]

    def __getitem__(self, index):
        return torch.LongTensor(self.x_sequence[index]), torch.LongTensor(self.x_candidate[index]), torch.LongTensor(self.x_sparse[index]), self.y[index]

    def __len__(self):
        return self.length





import numpy as np
import pandas as pd
from torch.utils.data import Dataset
from tqdm import tqdm
from sklearn.preprocessing import LabelEncoder
import torch

class CriteoDataset(Dataset):
    """
    Criteo Display Advertising Challenge Dataset

    :param dataset_path: criteo train.csv path.
    Reference:
        https://labs.criteo.com/2014/02/kaggle-display-advertising-challenge-dataset
        https://www.csie.ntu.edu.tw/~r01922136/kaggle-2014-criteo.pdf
    """

    def __init__(self, data_mode="mini", dataset_path="./criteo_sample_50w.csv"):
        if data_mode=="full":
            sparse_features = ['C' + str(i) for i in range(1, 27)]
            dense_features = ['I' + str(i) for i in range(1, 14)]
            target_columns = ['label']
            columns = target_columns + dense_features + sparse_features
            data = pd.read_csv(dataset_path, sep='\t', names = columns)
        elif data_mode=="mini":
            data = pd.read_csv(dataset_path)
            dense_features = [f for f in data.columns.tolist() if f[0] == "I"]
            sparse_features = [f for f in data.columns.tolist() if f[0] == "C"]
        
        data[sparse_features] = data[sparse_features].fillna('-10086', ) #一定要padding负数，Embedding层默认使用0作为padding值
        data[dense_features] = data[dense_features].fillna(0, )


        # ## 数值特征离散化
        for feat in tqdm(dense_features):
            data[feat] = data[feat].apply(lambda x:self.convert_numeric_feature(x))
        
        ## 数值特征标准化
        # for feat in tqdm(dense_features):
        #     mean = data[feat].mean()
        #     std = data[feat].std()
        #     data[feat] = (data[feat] - mean) / (std + 1e-12)   # 防止除零

        ## 类别特征labelencoder
        sparse_features = sparse_features + dense_features
        for feat in tqdm(sparse_features):
            lbe = LabelEncoder()
            data[feat] = lbe.fit_transform(data[feat]) #会先按从小到大排序，再编码

        #features = dense_features + sparse_features
        field_dims = np.zeros(len(sparse_features), dtype=np.uint32)
        for i, fea in enumerate(sparse_features):
           field_dims[i] = data[fea].nunique()
        
        self.sparse_field_dims = field_dims #每个sparse特征的unique值个数 list
        self.dense_field_nums = len(dense_features)
        self.length = data.shape[0]
        
        self.x_dense = data[dense_features].values 
        self.x_sparse = data[sparse_features].values 
        #self.x_sequence = data[features].values 

        self.y = data["label"].values

    def __getitem__(self, index):
        return torch.FloatTensor(self.x_dense[index]), torch.LongTensor(self.x_sparse[index]), self.y[index]

    def __len__(self):
        return self.length

    def convert_numeric_feature(self, val):
        v = int(val)
        if v > 2:
            return int(np.log(v) ** 2)
        else:
            return v - 2



import numpy as np
import torch
import torch.nn.functional as F
from torch.nn import ModuleList, Sequential, Linear, Embedding, BatchNorm1d, ReLU, Dropout
from .activation import activation_layer

class FeaturesLinear(torch.nn.Module):
    """
        Logistic Regression
    """
    def __init__(self, num_fields):
        super().__init__()
        self.fc = Linear(num_fields, 1, bias=True) 

    def forward(self, x):
        """
        :param x: Long tensor of size ``(batch_size, 1)``
        """
        #return self.fc(x)
        return torch.sigmoid(self.fc(x))


class FeaturesEmbedding(torch.nn.Module):
    """
        所有sparse特征 统一emb维度=embed_dim
    """

    def __init__(self, field_dims, embed_dim):
        super().__init__()
        self.embedding_list = ModuleList([
            Embedding(field_dims[i], embed_dim) for i in range(len(field_dims))]) 
        for emb in self.embedding_list:
            torch.nn.init.xavier_uniform_(emb.weight.data)

    def forward(self, x):
        """
        :param x: Long tensor of size ``(batch_size, num_fields)``
        :return: ``(batch_size, num_fields, embed_dim)`` 
        """
        sparse_emb = [emb(x[:,i].unsqueeze(1)) for i, emb in enumerate(self.embedding_list)]
        return torch.cat(sparse_emb, dim=1)

class MaskedAveragePooling(torch.nn.Module):
    def __init__(self):
        super(MaskedAveragePooling, self).__init__()

    def forward(self, embedding_matrix):
        """
        :param x: matrix tensor of size ``(batch_size, seq_length, embed_dim)``
        :return ``(batch_size, embed_dim)``
        """
        sum_pooling_matrix = torch.sum(embedding_matrix, dim=1)
        non_padding_length = (embedding_matrix != 0).sum(dim=1)
        embedding_vec = sum_pooling_matrix / (non_padding_length.float() + 1e-16)
        return embedding_vec # batch_size, 1, embedding_dim

class MaskedSumPooling(torch.nn.Module):
    def __init__(self):
        super(MaskedSumPooling, self).__init__()

    def forward(self, embedding_matrix):
        """
        :param x: matrix tensor of size ``(batch_size, seq_length, embed_dim)``
        :return ``(batch_size, embed_dim)``
        """
        return torch.sum(embedding_matrix, dim=1)

class SequenceFeaturesEmbedding(torch.nn.Module):
    """
        sequence embedding
        TODO：specific sparse fields shared embedding with specific sequence embedding 
    """

    def __init__(self, field_dims, embed_dim, mode="mean"):
        super().__init__()
        assert mode in ['concat', 'sum', 'mean'], "mode must in {'concat', 'sum', 'mean'}"
        self.field_dims = field_dims
        self.mode = mode
        if self.mode == "mean":
            self.pool = MaskedAveragePooling()
        elif self.mode == "sum":
            self.pool = MaskedSumPooling()
        elif self.mode == "concat":
            self.pool = None

        self.embedding_list = ModuleList([
            Embedding(field_dims[i], embed_dim) for i in range(len(field_dims))]) 
        for emb in self.embedding_list:
            torch.nn.init.xavier_uniform_(emb.weight.data)

    def forward(self, x, x_candidate=None):
        """
        :param x: Long tensor of size ``(batch_size, num_seq_fields, seq_length)``
        :param x_candidate: Long tensor of size ``(batch_size, num_seq_fields)``
                            for DIN model
        :return: ``(batch_size, num_seq_fields, emb_dim)`` 
                 ``(batch_size, num_seq_fields, seq_length, emb_dim) ``
        """
        if self.pool != None: #(batch_size, num_seq_fields, emb_dim) 
            sequence_emb = [self.pool(emb(x[:,i])).unsqueeze(1) for i, emb in enumerate(self.embedding_list)]
        else: #(batch_size, num_seq_fields, seq_length, emb_dim) 
            sequence_emb = [emb(x[:,i].unsqueeze(1)) for i, emb in enumerate(self.embedding_list)]
        
        sequence_emb = torch.cat(sequence_emb, dim=1) #concat all sequence fields
        
        if x_candidate != None: #(batch_size, num_seq_fields, emb_dim) 
            candidate_emb = [emb(x_candidate[:,i].unsqueeze(1)) for i, emb in enumerate(self.embedding_list)]
            candidate_emb = torch.cat(candidate_emb, dim=1)
            return sequence_emb, candidate_emb
        else:
            return sequence_emb

class MultiLayerPerceptron(torch.nn.Module):

    def __init__(self, input_dim, embed_dims, dropout=0, activation="relu", output_layer=True):
        super().__init__()
        layers = list()
        for embed_dim in embed_dims:
            layers.append(Linear(input_dim, embed_dim))
            layers.append(BatchNorm1d(embed_dim))
            layers.append(activation_layer(activation))
            layers.append(Dropout(p=dropout))
            input_dim = embed_dim
        if output_layer:
            layers.append(Linear(input_dim, 1))
        self.mlp = Sequential(*layers)

    def forward(self, x):
        """
        :param x: Float tensor of size ``(batch_size, embed_dim)``
        """
        return self.mlp(x)

class FactorizationMachine(torch.nn.Module):

    def __init__(self, reduce_sum=True):
        super().__init__()
        self.reduce_sum = reduce_sum

    def forward(self, x):
        """
        :param x: Float tensor of size ``(batch_size, num_fields, embed_dim)``
        """
        square_of_sum = torch.sum(x, dim=1) ** 2
        sum_of_square = torch.sum(x ** 2, dim=1)
        ix = square_of_sum - sum_of_square
        if self.reduce_sum:
            ix = torch.sum(ix, dim=1, keepdim=True)
        return 0.5 * ix


"""
Created on April 22, 2022
Reference: "DeepFM: A Factorization-Machine based Neural Network for CTR Prediction", IJCAI, 2017
@author: Mincai Lai, laimincai@shanghaitech.edu.cn
"""

import torch

from ..layers import FactorizationMachine, FeaturesEmbedding, FeaturesLinear, MultiLayerPerceptron


class DeepFM(torch.nn.Module):
    """
        Deep Factorization Machine Model
    """

    def __init__(self, dense_field_nums, sparse_field_dims, embed_dim, mlp_dims, dropout):
        super().__init__()
        self.linear = FeaturesLinear(dense_field_nums)
        self.fm = FactorizationMachine(reduce_sum=True)
        self.embedding = FeaturesEmbedding(sparse_field_dims, embed_dim)
        self.mlp = MultiLayerPerceptron(len(sparse_field_dims) * embed_dim, mlp_dims, dropout)

    def forward(self, x_dense, x_sparse):
        """
        :param x_dense: Long tensor of size ``(batch_size, num_dense_fields)``
        :param x_sparse: Long tensor of size ``(batch_size, num_sparse_fields)``
        """
        y_linear = self.linear(x_dense) #dense特征不参与交叉，建议提前离散化
        embed_x = self.embedding(x_sparse)
        y_fm = self.fm(embed_x)
        y_deep = self.mlp(embed_x.flatten(start_dim=1))
        y = y_linear + y_fm + y_deep
        return torch.sigmoid(y.squeeze(1))



import numpy as np
import pandas as pd
from sklearn.preprocessing import LabelEncoder
from torch.utils.data import Dataset
import torch

class TaobaoDataset(Dataset):
    """
    Criteo Display Advertising Challenge Dataset

    :param dataset_path: criteo train.csv path.
    Reference:
        https://labs.criteo.com/2014/02/kaggle-display-advertising-challenge-dataset
        https://www.csie.ntu.edu.tw/~r01922136/kaggle-2014-criteo.pdf
    """

    def __init__(self, data_path="./taobao_train.csv"):
        data = pd.read_csv(data_path) #sequence特征已padding 对齐长度50
        self.max_len = 50
        n_users = 864915 
        n_items = 4027558 
        n_cates = 9377
        sparse_features = ["user_id", "hist_len","user_count","item_count"]
        sequence_features = ["hist_item_id","hist_cate_id"]
        candidate_features = ["item_id","cate_id"]

        for fea in sparse_features:
            if fea!="user_id": #原始数据 user_id 已经encode
                le = LabelEncoder()
                data[fea] = le.fit_transform(data[fea])
        sparse_field_dims = np.zeros(len(sparse_features), dtype=np.int32)
        for i, fea in enumerate(sparse_features):
           n_unique = data[fea].nunique()
           print("feature: %s n_unique: %d"%(fea, n_unique))
           sparse_field_dims[i] = n_unique

        sequence_field_dims = np.zeros(len(sequence_features), dtype=np.int32)
        #sequence特征与它对应的candidate特征的embedding词表大小一致
        sequence_field_dims[0] = n_items
        sequence_field_dims[1] = n_cates
        
        self.sparse_field_dims = sparse_field_dims #每个sparse特征的unique值个数 list
        self.sequence_field_dims = sequence_field_dims
        self.length = data.shape[0]
        
        self.x_sequence = np.zeros((self.length, len(sequence_features),self.max_len),dtype=np.int32)

        for i in range(len(sequence_features)): #str->list
            seq_list = data[sequence_features[i]].apply(lambda x:eval(x))
            self.x_sequence[:,i,:] = np.array(seq_list.to_list(),dtype=np.int32)
            #(batch_size, num_seq_fields, seq_length)
 
        self.x_sparse = data[sparse_features].values 
        self.x_candidate = data[candidate_features].values 
        self.y = data["label"].values

    def __getitem__(self, index):
        return torch.LongTensor(self.x_sequence[index]), torch.LongTensor(self.x_candidate[index]), torch.LongTensor(self.x_sparse[index]), self.y[index]

    def __len__(self):
        return self.length




import sys
sys.path.append('/home/wlsh/ssd/lmc/torch-ctr')

import torch
import tqdm
from sklearn.metrics import roc_auc_score
from torch.utils.data import DataLoader
from torch_ctr.models import WideAndDeep, DeepFM, DIN

from taobao import TaobaoDataset
from criteo import CriteoDataset
from amazon import AmazonDataset

class EarlyStopper(object):

    def __init__(self, num_trials, save_path):
        self.num_trials = num_trials
        self.trial_counter = 0
        self.best_accuracy = 0
        self.save_path = save_path

    def is_continuable(self, model, accuracy):
        if accuracy > self.best_accuracy:
            self.best_accuracy = accuracy
            self.trial_counter = 0
            torch.save(model, self.save_path)
            return True
        elif self.trial_counter + 1 < self.num_trials:
            self.trial_counter += 1
            return True
        else:
            return False


def train(model, optimizer, scheduler, data_loader, criterion, device, log_interval=100):
    print("Current lr : {}".format(optimizer.state_dict()['param_groups'][0]['lr']))
    model.train()
    total_loss = 0
    tk0 = tqdm.tqdm(data_loader, smoothing=0, mininterval=1.0)
    #for i, (dense_fields, sparse_fields, target) in enumerate(tk0):
    #    dense_fields, sparse_fields, target = dense_fields.to(device), sparse_fields.to(device), target.to(device)
    for i, (sequence_fields, candidate_fields, sparse_fields, target) in enumerate(tk0):
        sequence_fields, candidate_fields, sparse_fields, target = \
            sequence_fields.to(device), candidate_fields.to(device),sparse_fields.to(device), target.to(device)
        #y = model(dense_fields, sparse_fields)
        y = model(sequence_fields, candidate_fields, sparse_fields)
        loss = criterion(y, target.float())
        model.zero_grad()
        loss.backward()
        optimizer.step()
        total_loss += loss.item()
        if (i + 1) % log_interval == 0:
            tk0.set_postfix(loss=total_loss / log_interval)
            total_loss = 0
    scheduler.step()

    

def test(model, data_loader, device):
    model.eval()
    targets, predicts = list(), list()
    with torch.no_grad():
        tk0 = tqdm.tqdm(data_loader, smoothing=0, mininterval=1.0)
        for i, (sequence_fields, candidate_fields, sparse_fields, target) in enumerate(tk0):
            sequence_fields, candidate_fields, sparse_fields, target = \
                sequence_fields.to(device), candidate_fields.to(device),sparse_fields.to(device), target.to(device)
            y = model(sequence_fields, candidate_fields, sparse_fields)
            targets.extend(target.tolist())
            predicts.extend(y.tolist())
    return roc_auc_score(targets, predicts)


def main(dataset_name,
         dataset_path,
         dataset_mode,
         model_name,
         epoch,
         learning_rate,
         batch_size,
         emb_dim,
         weight_decay,
         device,
         save_dir, 
         seed):
    torch.manual_seed(seed)
    device = torch.device(device)
    if dataset_name == "criteo":
        dataset = CriteoDataset(dataset_mode, dataset_path)
        model = get_model(model_name, dataset, emb_dim)

    elif dataset_name == "taobao":
        dataset = TaobaoDataset(dataset_path)
        model = DIN(dataset.sequence_field_dims, dataset.sparse_field_dims, embed_dim=emb_dim, mlp_dims=[256, 128], dropout=0.2)
    elif dataset_name == "amazon":
        dataset = AmazonDataset(dataset_path)
        model = DIN(dataset.sequence_field_dims, dataset.sparse_field_dims, embed_dim=emb_dim, mlp_dims=[256, 128], dropout=0.2)

    train_length = int(len(dataset) * 0.8)
    valid_length = int(len(dataset) * 0.1)
    test_length = len(dataset) - train_length - valid_length
    train_dataset, valid_dataset, test_dataset = torch.utils.data.random_split(
        dataset, (train_length, valid_length, test_length))
    
    train_data_loader = DataLoader(train_dataset, batch_size=batch_size, num_workers=8)
    valid_data_loader = DataLoader(valid_dataset, batch_size=batch_size, num_workers=8)
    test_data_loader = DataLoader(test_dataset, batch_size=batch_size, num_workers=8)
    
    model.to(device)
    criterion = torch.nn.BCELoss()
    optimizer = torch.optim.Adam(params=model.parameters(), lr=learning_rate, weight_decay=weight_decay)
    
    scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=40, gamma=0.1)
    #scheduler = torch.optim.lr_scheduler.ExponentialLR(optimizer, gamma=0.1)

    early_stopper = EarlyStopper(num_trials=50, save_path=f'{save_dir}/{model_name}.pth')
    print("train %s"%model_name)
    for epoch_i in range(epoch):
        train(model, optimizer, scheduler, train_data_loader, criterion, device)
        auc = test(model, valid_data_loader, device)
        print('epoch:', epoch_i, 'validation: auc:', auc)
        if not early_stopper.is_continuable(model, auc):
            print(f'validation: best auc: {early_stopper.best_accuracy}')
            break
    model = torch.load(f'{save_dir}/{model_name}.pth') #load best model
    auc = test(model, test_data_loader, device)
    print(f'test auc: {auc}')


def get_model(model_name, dataset, emb_dim):
    if model_name == "widedeep":
        model = WideAndDeep(dataset.dense_field_nums, dataset.sparse_field_dims, embed_dim=emb_dim, mlp_dims=(256, 128), dropout=0.2)
    elif model_name == "deepfm":
        model = DeepFM(dataset.dense_field_nums, dataset.sparse_field_dims, embed_dim=emb_dim, mlp_dims=(256, 128), dropout=0.2)
    return model

if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('--dataset_name', default='criteo')
    parser.add_argument('--dataset_path', default='./criteo/data/criteo_sample_50w.csv')
    parser.add_argument('--dataset_mode', default='mini')
    parser.add_argument('--model_name', default='din')
    parser.add_argument('--epoch', type=int, default=100)
    parser.add_argument('--learning_rate', type=float, default=5e-3)
    parser.add_argument('--batch_size', type=int, default=4096)
    parser.add_argument('--emb_dim', type=int, default=8)
    parser.add_argument('--weight_decay', type=float, default=1e-3)
    parser.add_argument('--device', default='cuda:0')
    parser.add_argument('--save_dir', default='chkpt')
    parser.add_argument('--seed', type=int, default=2022)

    args = parser.parse_args()
    main(args.dataset_name,
         args.dataset_path,
         args.dataset_mode,
         args.model_name,
         args.epoch,
         args.learning_rate,
         args.batch_size,
         args.emb_dim,
         args.weight_decay,
         args.device,
         args.save_dir,
         args.seed)